/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { SnackBarE2EComponent } from 'apps/alfa-e2e/src/components/ui/snackbar.e2e.component';
import { VorgangE2E, VorgangStatusE2E } from 'apps/alfa-e2e/src/model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from 'apps/alfa-e2e/src/page-objects/main.po';
import { PostfachMailPage } from 'apps/alfa-e2e/src/page-objects/postfach-mail.component.po';
import { visitUrl } from 'apps/alfa-e2e/src/support/cypress-helper';
import { contains, exist, notExist } from 'apps/alfa-e2e/src/support/cypress.util';
import { MessagesE2E } from 'apps/alfa-e2e/src/support/messages';
import { createPostfachUriByVorgangId } from 'apps/alfa-e2e/src/support/postfach-util';
import { UserRoleE2E, loginAsPeter } from 'apps/alfa-e2e/src/support/user-util';
import { buildVorgang, initVorgaenge } from 'apps/alfa-e2e/src/support/vorgang-util';

describe('Postfach Nachricht should be authorized by role', () => {
  const mainPage: MainPage = new MainPage();
  const snackbar: SnackBarE2EComponent = mainPage.getSnackBar();

  const postfachNachrichtPage: PostfachMailPage = new PostfachMailPage();

  describe(`for user Peter with role ${UserRoleE2E.VERWALTUNG_POSTSTELLE}`, () => {
    const vorgangInStatusNeuId: string = '601bc78eabc134051570aab8';
    const vorgangInStatusNeuUrl: string = createPostfachUriByVorgangId(vorgangInStatusNeuId);
    const vorgangInStatusNeu: VorgangE2E = {
      ...buildVorgang(vorgangInStatusNeuId, 'VorgangInStatusNeu'),
      status: VorgangStatusE2E.NEU,
    };

    const vorgangInStatusInBearbeitungId = '60250b9d383c182943f6ba79';
    const vorgangInStatusInBearbeitungUrl: string = createPostfachUriByVorgangId(vorgangInStatusInBearbeitungId);
    const vorgangInStatusInBearbeitung: VorgangE2E = {
      ...buildVorgang(vorgangInStatusInBearbeitungId, 'vorgangInStatusInBearbeitung'),
      status: VorgangStatusE2E.IN_BEARBEITUNG,
    };

    const vorgangInStatusAbgeschlossenId: string = '602566a807bb665df9a86e87';
    const vorgangInStatusAbgeschlossenUrl: string = createPostfachUriByVorgangId(vorgangInStatusAbgeschlossenId);
    const vorgangInStatusAbgeschlossen: VorgangE2E = {
      ...buildVorgang(vorgangInStatusAbgeschlossenId, 'vorgangInStatusAbgeschlossen'),
      status: VorgangStatusE2E.ABGESCHLOSSEN,
    };

    const vorgangInStatusBeschiedenId: string = '602566a207bb665df9a86e86';
    const vorgangInStatusBeschiedenUrl: string = createPostfachUriByVorgangId(vorgangInStatusBeschiedenId);
    const vorgangInStatusBeschieden: VorgangE2E = {
      ...buildVorgang(vorgangInStatusBeschiedenId, 'vorgangInStatusBeschieden'),
      status: VorgangStatusE2E.BESCHIEDEN,
    };

    beforeEach(() => {
      initVorgaenge([vorgangInStatusNeu, vorgangInStatusInBearbeitung, vorgangInStatusAbgeschlossen, vorgangInStatusBeschieden]);

      loginAsPeter();
    });

    it(`should show vorgang in Stauts ${VorgangStatusE2E.NEU}`, () => {
      visitUrl(vorgangInStatusNeuUrl);
      waitForSpinnerToDisappear();

      exist(postfachNachrichtPage.getRoot());
    });

    describe('should hide vorgang in status', () => {
      it(`${VorgangStatusE2E.IN_BEARBEITUNG}`, () => {
        visitUrl(vorgangInStatusInBearbeitungUrl);
        waitForSpinnerToDisappear();

        notExist(postfachNachrichtPage.getRoot());
        contains(snackbar.getMessage(), MessagesE2E.HTTP_STATUS_FORBIDDEN);
      });

      it(`${VorgangStatusE2E.ABGESCHLOSSEN}`, () => {
        visitUrl(vorgangInStatusAbgeschlossenUrl);
        waitForSpinnerToDisappear();

        notExist(postfachNachrichtPage.getRoot());
        contains(snackbar.getMessage(), MessagesE2E.HTTP_STATUS_FORBIDDEN);
      });

      it(`${VorgangStatusE2E.BESCHIEDEN}`, () => {
        visitUrl(vorgangInStatusBeschiedenUrl);
        waitForSpinnerToDisappear();

        notExist(postfachNachrichtPage.getRoot());
        contains(snackbar.getMessage(), MessagesE2E.HTTP_STATUS_FORBIDDEN);
      });
    });
  });
});
