/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BinaryFileService } from '@alfa-client/binary-file-shared';
import { CommandOrder, CommandResource, CommandService, CreateCommand } from '@alfa-client/command-shared';
import { NavigationService } from '@alfa-client/navigation-shared';
import { createEmptyStateResource, createStateResource, EMPTY_STRING, StateResource } from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { VorgangService, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { faker } from '@faker-js/faker';
import { expect } from '@jest/globals';
import { ResourceUri } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { CommandLinkRel } from 'libs/command-shared/src/lib/command.linkrel';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createKommentar, createKommentarListResource, createKommentarResource } from 'libs/kommentar-shared/test/kommentar';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { of, Subscription } from 'rxjs';
import { singleCold } from '../../../tech-shared/test/marbles';
import { KommentarLinkRel, KommentarListLinkRel } from './kommentar.linkrel';
import { Kommentar, KOMMENTAR_UPLOADED_ATTACHMENTS, KommentarListResource, KommentarResource } from './kommentar.model';
import { KommentarRepository } from './kommentar.repository';
import { KommentarService } from './kommentar.service';

describe('KommentarService', () => {
  let service: KommentarService;
  let repository: Mock<KommentarRepository>;
  let commandService: Mock<CommandService>;
  let navigationService: Mock<NavigationService>;
  let vorgangService: Mock<VorgangService>;
  let binaryFileService: Mock<BinaryFileService>;

  const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource();
  const vorgangWithEingangStateResource: StateResource<VorgangWithEingangResource> = createStateResource(vorgang);

  const kommentarList: KommentarListResource = createKommentarListResource();
  const kommentar: Kommentar = createKommentar();
  const kommentarResource: KommentarResource = createKommentarResource();

  const command: CommandResource = createCommandResource();
  const commandStateResource: StateResource<CommandResource> = createStateResource(command);

  beforeEach(() => {
    repository = mock(KommentarRepository);
    commandService = mock(CommandService);
    navigationService = mock(NavigationService);
    vorgangService = {
      ...mock(VorgangService),
      selectVorgangWithEingang: jest.fn().mockReturnValue(of(vorgangWithEingangStateResource)),
    };
    binaryFileService = mock(BinaryFileService);

    navigationService.urlChanged.mockReturnValue(of({}));

    service = new KommentarService(
      useFromMock(repository),
      useFromMock(commandService),
      useFromMock(navigationService),
      useFromMock(vorgangService),
      useFromMock(binaryFileService),
    );
  });

  describe('getKommentareByVorgang', () => {
    beforeEach(() => {
      repository.findKommentare.mockReturnValue(of(kommentarList));

      service._setListLoadingTrue = jest.fn();
      service._setKommentarList = jest.fn();
      service._refreshVorgangSubscription = jest.fn();
    });

    it('should call refresh vorgang subscription', () => {
      service.getKommentareByVorgang(vorgang);

      expect(service._refreshVorgangSubscription).toHaveBeenCalled();
    });

    it('should set loading to true', () => {
      service.getKommentareByVorgang(vorgang);

      expect(service._setListLoadingTrue).toHaveBeenCalled();
    });

    it('should call repository', () => {
      service.getKommentareByVorgang(vorgang).subscribe();

      expect(repository.findKommentare).toHaveBeenCalledWith(vorgang);
    });

    it('should set loaded resource', () => {
      service.getKommentareByVorgang(vorgang);

      expect(service._setKommentarList).toHaveBeenCalledWith(kommentarList);
    });
  });

  describe('refresh vorgang subscription', () => {
    beforeEach(() => {
      service.vorgangSubscription = <any>mock(Subscription);
      service._listenToVorgangChange = jest.fn();
    });

    it('should unsubscribe existing subscription', () => {
      service._refreshVorgangSubscription();

      expect(service.vorgangSubscription.unsubscribe).toHaveBeenCalled();
    });
    it('should call listen to vorgang change', () => {
      service._refreshVorgangSubscription();

      expect(service._listenToVorgangChange).toHaveBeenCalled();
    });
  });

  describe('createKommentar', () => {
    describe('init and next value', () => {
      beforeEach(() => {
        commandService.createCommand.mockReturnValue(hot('-a', { a: commandStateResource }));
      });

      it('should be return', () => {
        const result = service.createKommentar(kommentar);

        expect(result).toBeObservable(cold('ab', { a: createEmptyStateResource(true), b: commandStateResource }));
      });
    });

    describe('further process', () => {
      beforeEach(() => {
        commandService.createCommand.mockReturnValue(of(commandStateResource));
      });

      it('should call command service', () => {
        service.createKommentar(kommentar);

        expect(commandService.createCommand).toHaveBeenCalledWith(
          service.kommentarList$.value.resource,
          KommentarListLinkRel.CREATE_KOMMENTAR,
          service._createCreateKommentarCommand(kommentar),
        );
      });

      it('should call getEffectedResource', () => {
        service._afterCreateOrEditKommentar = jest.fn();

        service.createKommentar(kommentar).subscribe();

        expect(service._afterCreateOrEditKommentar).toHaveBeenCalledWith(commandStateResource);
      });

      it('should call createCreateKommentarCommand', () => {
        service._createCreateKommentarCommand = jest.fn();

        service.createKommentar(kommentar);

        expect(service._createCreateKommentarCommand).toHaveBeenCalledWith(kommentar);
      });

      it('should create CreateKommentarCommand', () => {
        var command: CreateCommand = service._createCreateKommentarCommand(kommentar);

        expect(command).toEqual({ order: CommandOrder.CREATE_KOMMENTAR, body: kommentar });
      });
    });
  });

  describe('edit kommentar', () => {
    describe('init and next value', () => {
      beforeEach(() => {
        commandService.createCommand.mockReturnValue(hot('-a', { a: commandStateResource }));
      });

      it('should be return', () => {
        const result = service.editKommentar(kommentarResource, kommentar);

        expect(result).toBeObservable(cold('ab', { a: createEmptyStateResource(true), b: commandStateResource }));
      });
    });

    describe('further process', () => {
      beforeEach(() => {
        commandService.createCommand.mockReturnValue(of(commandStateResource));
      });

      it('should call command service', () => {
        service.editKommentar(kommentarResource, kommentar);

        expect(commandService.createCommand).toHaveBeenCalledWith(
          kommentarResource,
          KommentarLinkRel.EDIT,
          service._createEditKommentarCommand(kommentar),
        );
      });

      it('should call getEffectedResource', () => {
        service._afterCreateOrEditKommentar = jest.fn();

        service.editKommentar(kommentarResource, kommentar).subscribe();

        expect(service._afterCreateOrEditKommentar).toHaveBeenCalledWith(commandStateResource);
      });

      it('should call createEditKommentarCommand', () => {
        service._createEditKommentarCommand = jest.fn();

        service.editKommentar(kommentarResource, kommentar);

        expect(service._createEditKommentarCommand).toHaveBeenCalledWith(kommentar);
      });

      it('should create CreateKommentarCommand', () => {
        var command: CreateCommand = service._createEditKommentarCommand(kommentar);

        expect(command).toEqual({ order: CommandOrder.EDIT_KOMMENTAR, body: kommentar });
      });
    });
  });

  describe('afterCreateOrEditKommentar', () => {
    describe('on command is done', () => {
      const kommentarList: KommentarListResource = createKommentarListResource();

      beforeEach(() => {
        commandService.getEffectedResource.mockReturnValue(of(kommentarList));
      });

      it('should hide formular', () => {
        service.hideFormular = jest.fn();

        service._afterCreateOrEditKommentar({
          ...commandStateResource,
          resource: createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]),
        });

        expect(service.hideFormular).toHaveBeenCalled();
      });

      it('should reload vorgang', () => {
        service._afterCreateOrEditKommentar({
          ...commandStateResource,
          resource: createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]),
        });

        expect(vorgangService.reloadCurrentVorgang).toHaveBeenCalled();
      });

      it('should set kommentar list on reload', () => {
        service._setKommentarListReload = jest.fn();

        service._afterCreateOrEditKommentar({
          ...commandStateResource,
          resource: createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]),
        });

        expect(service._setKommentarListReload).toHaveBeenCalled();
      });
    });

    describe('on command is pending', () => {
      it('should do nothing', () => {
        service._afterCreateOrEditKommentar({
          ...commandStateResource,
          resource: createCommandResource([CommandLinkRel.UPDATE]),
        });

        expect(commandService.getEffectedResource).not.toHaveBeenCalled();
      });
    });
  });

  describe('onNavigation', () => {
    beforeEach(() => {
      service.clearUploadedFiles = jest.fn();
      service.hideFormular = jest.fn();
    });

    it('should clear uploaded files', () => {
      service._onNavigation({});

      expect(service.clearUploadedFiles).toHaveBeenCalled();
    });

    it('should hide forumlar', () => {
      service._onNavigation({});

      expect(service.hideFormular).toHaveBeenCalled();
    });

    describe('to vorgang list', () => {
      it('should set reload flag of kommentar list', () => {
        service.kommentarList$.next(createEmptyStateResource());

        service._onNavigation({});

        expect(service.kommentarList$.value.reload).toEqual(true);
      });
    });

    describe('to vorgang detail page', () => {
      it('should set kommentar list to reload', () => {
        service.kommentarList$.next(createEmptyStateResource());

        service._onNavigation({ vorgangWithEingangUrl: 'X' });

        expect(service.kommentarList$.value.reload).toEqual(true);
      });
    });
  });

  describe('attachments', () => {
    it('should be loaded if link avaible', () => {
      const kommentarResource = createKommentarResource([KommentarLinkRel.ATTACHMENTS]);
      service._getAttachments(kommentarResource);

      expect(binaryFileService.getFiles).toHaveBeenCalledWith(kommentarResource, KommentarLinkRel.ATTACHMENTS);
    });

    it('should not be loaded if no link available', () => {
      service._getAttachments(kommentarResource);

      expect(binaryFileService.getFiles).not.toHaveBeenCalled();
    });
  });

  describe('canCreateNewKommentar', () => {
    it('should create new Kommentare', () => {
      const canCreateNewKommentar$ = cold('a', { a: true });

      const observable = service.canCreateNewKommentar(createKommentarListResource([KommentarListLinkRel.CREATE_KOMMENTAR]));

      expect(observable).toBeObservable(canCreateNewKommentar$);
    });

    it('should not create new Kommentare if formular visible', () => {
      const canCreateNewKommentar$ = cold('a', { a: false });
      service.formularVisibility$.next(true);

      const observable = service.canCreateNewKommentar(createKommentarListResource([KommentarListLinkRel.CREATE_KOMMENTAR]));

      expect(observable).toBeObservable(canCreateNewKommentar$);
    });

    it('should not create new Kommentare if link not exists', () => {
      const canCreateNewKommentar$ = cold('a', { a: false });
      service.formularVisibility$.next(false);

      const observable = service.canCreateNewKommentar(createKommentarListResource());

      expect(observable).toBeObservable(canCreateNewKommentar$);
    });
  });

  describe('clearUploadedFiles', () => {
    it('should call binary file service', () => {
      service.clearUploadedFiles();

      expect(binaryFileService.clearUploadedFiles).toHaveBeenCalledWith(KOMMENTAR_UPLOADED_ATTACHMENTS);
    });
  });

  describe('is new kommentar formular visible', () => {
    it('should emit true', () => {
      service._currentlyEdited$.next(EMPTY_STRING);
      service.formularVisibility$.next(true);

      expect(service.isFormularVisible()).toBeObservable(singleCold(true));
    });

    it('should emit false if any kommentar is being edited', () => {
      service._currentlyEdited$.next(faker.internet.url());
      service.formularVisibility$.next(true);

      expect(service.isFormularVisible()).toBeObservable(singleCold(false));
    });

    it('should emit false if not visible', () => {
      service._currentlyEdited$.next(EMPTY_STRING);
      service.formularVisibility$.next(false);

      expect(service.isFormularVisible()).toBeObservable(singleCold(false));
    });
  });

  describe('show new kommentar formular', () => {
    beforeEach(() => {
      service.setCurrentlyEdited = jest.fn();
    });

    it('should set currently edited to empty string', () => {
      service.showFormular();

      expect(service.setCurrentlyEdited).toHaveBeenCalledWith(EMPTY_STRING);
    });

    it('should set formular visibility', () => {
      service.showFormular();

      expect(service.formularVisibility$).toBeObservable(singleCold(true));
    });
  });

  describe('set currently edited kommentar uri', () => {
    beforeEach(() => {
      service.clearUploadedFiles = jest.fn();
      service.hideFormular = jest.fn();
    });

    it('should clear uploaded files', () => {
      service.setCurrentlyEdited(faker.internet.url());

      expect(service.clearUploadedFiles).toHaveBeenCalled();
    });

    it('should hide kommentar creation formular', () => {
      service.setCurrentlyEdited(faker.internet.url());

      expect(service.hideFormular).toHaveBeenCalled();
    });

    it('should NOT hide kommentar creation formular', () => {
      service.setCurrentlyEdited(EMPTY_STRING);

      expect(service.hideFormular).not.toHaveBeenCalled();
    });

    it('should emit currently edited resource uri', () => {
      const resourceUri: ResourceUri = faker.internet.url();

      service.setCurrentlyEdited(resourceUri);

      expect(service._currentlyEdited$).toBeObservable(singleCold(resourceUri));
    });
  });

  describe('listen to vorgang change', () => {
    beforeEach(() => {
      service._handleVorgangChange = jest.fn();
    });

    it('should call vorgang service to get vorgang with eingang', () => {
      service._listenToVorgangChange();

      expect(vorgangService.selectVorgangWithEingang).toHaveBeenCalled();
    });

    it('should call handle vorgang change', () => {
      service._listenToVorgangChange();

      expect(service._handleVorgangChange).toHaveBeenCalledWith(vorgangWithEingangStateResource);
    });
  });

  describe('handle vorgang change', () => {
    it('should set reload flag if list is loaded and state resource is loading', () => {
      service.shouldReload = false;
      service.kommentarList$.next(createStateResource(createKommentarListResource()));

      service._handleVorgangChange({ ...vorgangWithEingangStateResource, loading: true });

      expect(service.shouldReload).toBeTruthy();
    });

    it('should set reload flag if list is loaded and state resource is reloading', () => {
      service.kommentarList$.next(createStateResource(createKommentarListResource()));
      service.shouldReload = false;

      service._handleVorgangChange({ ...vorgangWithEingangStateResource, reload: true });

      expect(service.shouldReload).toBeTruthy();
    });

    describe('on setted reload flag', () => {
      beforeEach(() => {
        service._setKommentarListReload = jest.fn();
        service.shouldReload = true;
      });

      describe('and loaded vorgang resource', () => {
        it('should call set kommentar list reload', () => {
          service._handleVorgangChange(vorgangWithEingangStateResource);

          expect(service._setKommentarListReload).toHaveBeenCalled();
        });

        it('and loaded vorgang resource should call set kommentar list reload', () => {
          service._handleVorgangChange(vorgangWithEingangStateResource);

          expect(service.shouldReload).toBeFalsy();
        });
      });
    });
  });
});
