/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BinaryFileService } from '@alfa-client/binary-file-shared';
import { CommandResource, CommandService } from '@alfa-client/command-shared';
import { NavigationService } from '@alfa-client/navigation-shared';
import { createEmptyStateResource, createStateResource, StateResource } from '@alfa-client/tech-shared';
import { Mock, mock } from '@alfa-client/test-utils';
import { SnackBarService } from '@alfa-client/ui';
import { VorgangService, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { TestBed } from '@angular/core/testing';
import { MatDialog } from '@angular/material/dialog';
import { expect } from '@jest/globals';
import { CommandLinkRel } from 'libs/command-shared/src/lib/command.linkrel';
import { createCommandErrorResource, createCommandResource } from 'libs/command-shared/test/command';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { BehaviorSubject, of } from 'rxjs';
import {
  createPostfachFeatures,
  createPostfachMail,
  createPostfachMailListResource,
  createPostfachMailResource,
  createPostfachSettings,
} from '../../test/postfach';
import { PostfachFacade } from './+state/postfach.facade';
import { PostfachMailLinkRel, PostfachMailListLinkRel } from './postfach.linkrel';
import { PostfachMessages } from './postfach.message';
import { PostfachMail, PostfachMailListResource, PostfachMailResource } from './postfach.model';
import { PostfachRepository } from './postfach.repository';
import { PostfachService } from './postfach.service';
import { createSendPostfachMailCommand } from './postfach.util';

describe('PostfachService', () => {
  let service: PostfachService;

  let repository: Mock<PostfachRepository>;
  const commandService: Mock<CommandService> = mock(CommandService);
  const navigationService: Mock<NavigationService> = mock(NavigationService);
  let vorgangService: Mock<VorgangService>;
  const snackbarService: Mock<SnackBarService> = mock(SnackBarService);
  const dialog: Mock<MatDialog> = <Mock<MatDialog>>{ closeAll: jest.fn() };
  const postfachFacade: Mock<PostfachFacade> = mock(PostfachFacade);
  const binaryFileService: Mock<BinaryFileService> = mock(BinaryFileService);

  const urlChangedParams = {};

  const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource();
  const vorgangWithEingangStateResource: StateResource<VorgangWithEingangResource> = createStateResource(vorgang);

  beforeEach(() => {
    vorgangService = {
      ...mock(VorgangService),
      selectVorgangWithEingang: jest.fn().mockReturnValue(of(vorgangWithEingangStateResource)),
    };
    navigationService.urlChanged = jest.fn();
    navigationService.urlChanged.mockReturnValue(of(urlChangedParams));

    repository = mock(PostfachRepository);

    TestBed.configureTestingModule({
      providers: [
        { provide: CommandService, useValue: commandService },
        { provide: NavigationService, useValue: navigationService },
        { provide: VorgangService, useValue: vorgangService },
        { provide: SnackBarService, useValue: snackbarService },
        { provide: MatDialog, useValue: dialog },
        { provide: PostfachFacade, useValue: postfachFacade },
        { provide: BinaryFileService, useValue: binaryFileService },
        { provide: PostfachRepository, useValue: repository },
        PostfachService,
      ],
    });

    service = TestBed.inject(PostfachService);
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  describe('send message', () => {
    const postfachMail: PostfachMail = createPostfachMail();
    const commandStateResource: StateResource<CommandResource> = createStateResource(createCommandResource());

    beforeEach(() => {
      commandService.createCommand.mockReturnValue(of(commandStateResource));
      vorgangService.getPendingSendPostfachMailCommand.mockReturnValue(of(commandStateResource));
    });

    it('should call vorgang service and set command on loading', () => {
      service.sendMail(postfachMail);

      expect(vorgangService.setPendingSendPostfachMailSingleCommandLoading).toHaveBeenCalled();
    });

    it('should call command service', () => {
      service.sendMail(postfachMail);

      expect(commandService.createCommand).toHaveBeenCalled();
    });

    it('should call vorgang service and get command', () => {
      service.sendMail(postfachMail);

      expect(vorgangService.getPendingSendPostfachMailCommand).toHaveBeenCalled();
    });

    describe('reload postfach list', () => {
      beforeEach(() => {
        service._refreshPostfachMailList = jest.fn();
      });

      it('should execute on command is done', () => {
        const commandStateResource: StateResource<CommandResource> = createStateResource(
          createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]),
        );
        commandService.createCommand.mockReturnValue(of(commandStateResource));

        service.sendMail(postfachMail);

        expect(service._refreshPostfachMailList).toHaveBeenCalledWith(commandStateResource);
      });

      it('should not execute on pending command', () => {
        commandService.createCommand.mockReturnValue(of(createStateResource(createCommandResource([CommandLinkRel.UPDATE]))));

        service.sendMail(postfachMail);

        expect(service._refreshPostfachMailList).not.toHaveBeenCalled();
      });
    });
  });

  describe('sendExistingNachricht', () => {
    const postfachNachricht: PostfachMailResource = createPostfachMailResource();
    const nachricht: PostfachMail = createPostfachMail();

    const commandStateResource: StateResource<CommandResource> = createStateResource(createCommandResource());

    it('should call doSendNachricht', () => {
      service._doSendNachricht = jest.fn();
      commandService.createCommand.mockReturnValue(of(commandStateResource));
      vorgangService.getPendingSendPostfachMailCommand.mockReturnValue(of(commandStateResource));

      service.sendExistingMail(postfachNachricht, nachricht);

      expect(service._doSendNachricht).toHaveBeenCalledWith(
        postfachNachricht,
        PostfachMailLinkRel.SEND,
        createSendPostfachMailCommand(nachricht),
      );
    });
  });

  describe('resend message', () => {
    const postfachMailResource: PostfachMailResource = createPostfachMailResource();

    beforeEach(() => {
      service._handleSendPostfachMailIsDone = jest.fn();

      commandService.createCommand.mockReturnValue(of(createStateResource(createCommandResource())));
      vorgangService.getPendingSendPostfachMailCommand.mockReturnValue(of(createStateResource(createCommandResource())));
    });

    it('should call vorgang service and set command on loading', () => {
      service.resendMail(postfachMailResource);

      expect(vorgangService.setPendingSendPostfachMailSingleCommandLoading).toHaveBeenCalled();
    });

    it('should call command service', () => {
      service.resendMail(postfachMailResource);

      expect(commandService.createCommand).toHaveBeenCalled();
    });

    it('should call vorgang service and get command', () => {
      service.resendMail(postfachMailResource);

      expect(vorgangService.getPendingSendPostfachMailCommand).toHaveBeenCalled();
    });

    it('should NOT call handleSendPostfachMailIsDone if command is pending', () => {
      commandService.createCommand.mockReturnValue(of(createStateResource(createCommandResource([CommandLinkRel.UPDATE]))));
      service.resendMail(postfachMailResource);

      expect(service._handleSendPostfachMailIsDone).not.toHaveBeenCalled();
    });

    it('should call handleSendPostfachMailIsDone if command is done', () => {
      const commandResponse: StateResource<CommandResource> = createStateResource(
        createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]),
      );
      commandService.createCommand.mockReturnValue(of(commandResponse));

      service.resendMail(postfachMailResource);

      expect(service._handleSendPostfachMailIsDone).toHaveBeenCalledWith(commandResponse);
    });
  });

  describe('handleSendNachrichtCommand', () => {
    const commandStateResource: StateResource<CommandResource> = createStateResource(createCommandResource());

    beforeEach(() => {
      service._commandIsDone = jest.fn();
    });

    it('should call vorgang service', () => {
      service._handleSendNachrichtCommand(commandStateResource);

      expect(vorgangService.setPendingSendPostfachMailCommand).toHaveBeenCalledWith(commandStateResource);
    });

    it('should call commandIsDone', () => {
      service._handleSendNachrichtCommand(commandStateResource);

      expect(service._commandIsDone).toHaveBeenCalledWith(commandStateResource);
    });
  });

  describe('commandIsDone', () => {
    beforeEach(() => {
      service._handleSendPostfachMailIsDone = jest.fn();
    });

    it('should call handleSendPostfachMailIsDone', () => {
      const commandStateResource: StateResource<CommandResource> = createStateResource(
        createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]),
      );

      service._commandIsDone(commandStateResource);

      expect(service._handleSendPostfachMailIsDone).toHaveBeenCalledWith(commandStateResource);
    });

    describe('on command is done', () => {
      it('should call snackbar service', () => {
        const commandStateResource: StateResource<CommandResource> = createStateResource({
          ...createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]),
          errorMessage: null,
        });

        service._commandIsDone(commandStateResource);

        expect(snackbarService.show).toHaveBeenCalledWith(commandStateResource.resource, PostfachMessages.SEND_SUCCESSFUL);
      });
    });

    describe('on command is done with error', () => {
      it('should call snackbar service', () => {
        const commandStateResource: StateResource<CommandResource> = createStateResource(
          createCommandErrorResource([CommandLinkRel.EFFECTED_RESOURCE]),
        );

        service._commandIsDone(commandStateResource);

        expect(snackbarService.showError).toHaveBeenCalledWith(PostfachMessages.SEND_FAILED);
      });
    });
  });

  describe('listenToNavigation', () => {
    it('should call usnsubcribeToNavigation', () => {
      service._listenToNavigation();

      expect(navigationService.urlChanged).toHaveBeenCalled();
    });

    it('should call navigation service', () => {
      service._unsubscribeToNavigation = jest.fn();

      service._listenToNavigation();

      expect(service._unsubscribeToNavigation).toHaveBeenCalled();
    });

    it('should call onNavigation', () => {
      service._onNavigation = jest.fn();

      service._listenToNavigation();

      expect(service._onNavigation).toHaveBeenCalledWith(urlChangedParams);
    });
  });

  describe('onNavigation', () => {
    describe('to vorgang list', () => {
      beforeEach(() => {
        service._setPollingFalse = jest.fn();
        service._clearPostfachMailList = jest.fn();
        service._unsubscribe = jest.fn();
      });

      it('should set polling false', () => {
        service._onNavigation({});

        expect(service._setPollingFalse).toHaveBeenCalled();
      });

      it('should call clearPostfachMailList', () => {
        service._onNavigation({});

        expect(service._clearPostfachMailList).toHaveBeenCalled();
      });

      it('should call closeOpenDialogs', () => {
        service._onNavigation({});

        expect(dialog.closeAll).toHaveBeenCalled();
      });

      it('should call unsubscribe', () => {
        service._onNavigation({});

        expect(service._unsubscribe).toHaveBeenCalled();
      });
    });

    describe('to vorgang detail page', () => {
      it('should set postfach nachrichten list to reload', () => {
        service.postfachMailList$.next(createEmptyStateResource());

        service._onNavigation({ vorgangWithEingangUrl: 'X' });

        expect(service.postfachMailList$.value.reload).toEqual(true);
      });
    });

    describe('to postfach page', () => {
      beforeEach(() => {
        service._resetHasNewPostfachNachrichten = jest.fn();
        navigationService.isPostfachPage.mockReturnValue(true);
      });

      it('should call navigationService', () => {
        service._onNavigation({});

        expect(navigationService.isPostfachPage).toHaveBeenCalled();
      });

      it('should reset hasNewPostfachNachrichten', () => {
        navigationService.isPostfachPage.mockReturnValue(true);

        service._onNavigation({});

        expect(service._resetHasNewPostfachNachrichten).toHaveBeenCalled();
      });
    });
  });

  describe('getPendingSendPostfachMailCommand', () => {
    beforeEach(() => {
      service._unsubscribe = jest.fn();
      service._pollSendPostfachMailCommand = jest.fn();

      commandService.pollCommand.mockReturnValue(of(createStateResource(createCommandResource())));
      vorgangService.getPendingSendPostfachMailCommand.mockReturnValue(of(createStateResource(createCommandResource())));
    });

    it('should call navigation service', () => {
      service.getPendingSendPostfachMailCommand();

      expect(navigationService.urlChanged).toHaveBeenCalled();
    });
  });

  describe('resetHasNewPostfachNachrichten', () => {
    const postfachListNachrichtenResource: PostfachMailListResource = createPostfachMailListResource();

    it('should call doResetHasNewPostfachNachrichten', () => {
      service.getPostfachMailListByVorgang = jest.fn();
      service._doResetHasNewPostfachNachrichten = jest.fn();
      (<any>service.getPostfachMailListByVorgang).mockReturnValue(of(postfachListNachrichtenResource));

      service._resetHasNewPostfachNachrichten();

      expect(service._doResetHasNewPostfachNachrichten).toHaveBeenCalled();
    });
  });

  describe('doResetHasNewPostfachNachrichten', () => {
    describe('on existing link', () => {
      const postfachNachrichtenListResource: PostfachMailListResource = createPostfachMailListResource([
        PostfachMailListLinkRel.RESET_HAS_NEW_POSTFACH_NACHRICHT,
      ]);

      beforeEach(() => {
        service.postfachMailList$.next(createStateResource(postfachNachrichtenListResource));
        repository.resetHasNewPostfachNachrichten.mockReturnValue(of(postfachNachrichtenListResource));
      });

      it('should call repository if link exists', () => {
        service._doResetHasNewPostfachNachrichten();

        expect(repository.resetHasNewPostfachNachrichten).toHaveBeenCalledWith(postfachNachrichtenListResource);
      });
    });

    it('should NOT call repository if link not exists', () => {
      const postfachNachrichtenListResource: PostfachMailListResource = createPostfachMailListResource();
      service.postfachMailList$.next(createStateResource(postfachNachrichtenListResource));

      service._doResetHasNewPostfachNachrichten();

      expect(repository.resetHasNewPostfachNachrichten).not.toHaveBeenCalled();
    });
  });

  describe('getPostfachMailListByVorgang', () => {
    const postfachMailList: PostfachMailListResource = createPostfachMailListResource();

    beforeEach(() => {
      repository.loadPostfachMailList.mockReturnValue(of(postfachMailList));
      service._setPostfachMailListLoading = jest.fn();
      service._setPostfachMailList = jest.fn();
      vorgangService.getVorgangWithEingang.mockReturnValue(of(createStateResource(createVorgangWithEingangResource())));
      service.postfachMailList$.next(createEmptyStateResource());
      service._refreshVorgangSubscription = jest.fn();
    });

    it('should call refresh vorgang subscription', () => {
      service.getPostfachMailListByVorgang();

      expect(service._refreshVorgangSubscription).toHaveBeenCalled();
    });

    it('should set loading to true', () => {
      service.getPostfachMailListByVorgang();

      expect(service._setPostfachMailListLoading).toHaveBeenCalled();
    });

    it('should call vorgang service', () => {
      service.getPostfachMailListByVorgang();

      expect(vorgangService.getVorgangWithEingang).toHaveBeenCalled();
    });

    it('should call repository', () => {
      service.getPostfachMailListByVorgang().subscribe();

      expect(repository.loadPostfachMailList).toHaveBeenCalled();
    });

    it('should set loading to false', () => {
      service.getPostfachMailListByVorgang();

      expect(service._setPostfachMailList).toHaveBeenCalled();
    });
  });

  describe('handleSendPostfachMailIsDone', () => {
    const stateResource: StateResource<CommandResource> = createStateResource(createCommandResource());

    beforeEach(() => {
      service._setPollingFalse = jest.fn();
      service._setPostfachMailList = jest.fn();

      commandService.getEffectedResource.mockReturnValue(of(createCommandResource()));
    });

    it('should call command service to get effected resource', () => {
      service._handleSendPostfachMailIsDone(stateResource);

      expect(commandService.getEffectedResource).toHaveBeenCalledWith(stateResource.resource);
    });

    it('should set postfachMailList', () => {
      service._handleSendPostfachMailIsDone(stateResource);

      expect(service._setPostfachMailList).toHaveBeenCalled();
    });

    it('should set polling to false', () => {
      service._handleSendPostfachMailIsDone(stateResource);

      expect(service._setPollingFalse).toHaveBeenCalled();
    });
  });

  describe('isDownloadPdfInProgress', () => {
    const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource();

    it('should call facade to download pdf', (done) => {
      vorgangService.getVorgangWithEingang.mockReturnValue(of(createStateResource(vorgang)));
      postfachFacade.isDownloadPdfInProgress.mockReturnValue(of(true));

      service.isDownloadPdfInProgress().subscribe(() => {
        expect(postfachFacade.downloadPdf).toHaveBeenCalledWith(vorgang);
        done();
      });
    });

    describe('combine latest', () => {
      beforeEach(() => {
        vorgangService.getVorgangWithEingang.mockReturnValue(of(vorgang));
        postfachFacade.isDownloadPdfInProgress = jest.fn().mockReturnValue(true);
      });

      it('should call facade to get isDownloadPdfInProgress', () => {
        service.isDownloadPdfInProgress();

        expect(postfachFacade.isDownloadPdfInProgress).toHaveBeenCalled();
      });

      it('should call vorgang service to get vorgang', () => {
        service.isDownloadPdfInProgress();

        expect(vorgangService.getVorgangWithEingang).toHaveBeenCalled();
      });
    });
  });

  describe('downloadPdf', () => {
    it('should call facade', () => {
      service.downloadPdf();

      expect(postfachFacade.startDownloadPdf).toHaveBeenCalled();
    });
  });

  describe('getFeatures', () => {
    it('should return features by list stateResource', (done) => {
      service.postfachMailList$ = new BehaviorSubject(createStateResource(createPostfachMailListResource()));

      service.getFeatures().subscribe((features) => {
        expect(features).toEqual(createPostfachFeatures());
        done();
      });
    });
  });

  describe('getSettings', () => {
    it('should return settings by list stateResource', (done) => {
      service.postfachMailList$ = new BehaviorSubject(createStateResource(createPostfachMailListResource()));

      service.getSettings().subscribe((settings) => {
        expect(settings).toEqual(createPostfachSettings());
        done();
      });
    });
  });

  describe('close open dialogs', () => {
    beforeEach(() => {
      service.clearUploadedFiles = jest.fn();
    });

    it('should clear uploaded files', () => {
      service._closeOpenDialogs();

      expect(service.clearUploadedFiles).toHaveBeenCalled();
    });
  });

  describe('listen to vorgang change', () => {
    beforeEach(() => {
      service._handleVorgangChange = jest.fn();
    });

    it('should call vorgang service to get vorgang with eingang', () => {
      service._listenToVorgangChange();

      expect(vorgangService.selectVorgangWithEingang).toHaveBeenCalled();
    });

    it('should call handle vorgang change', () => {
      service._listenToVorgangChange();

      expect(service._handleVorgangChange).toHaveBeenCalledWith(vorgangWithEingangStateResource);
    });
  });

  describe('handle vorgang change', () => {
    it('should set reload flag if list is loaded and state resource is loading', () => {
      service.shouldReload = false;
      service.postfachMailList$.next(createStateResource(createPostfachMailListResource()));

      service._handleVorgangChange({ ...vorgangWithEingangStateResource, loading: true });

      expect(service.shouldReload).toBeTruthy();
    });

    it('should set reload flag if list is loaded and state resource is reloading', () => {
      service.postfachMailList$.next(createStateResource(createPostfachMailListResource()));
      service.shouldReload = false;

      service._handleVorgangChange({ ...vorgangWithEingangStateResource, reload: true });

      expect(service.shouldReload).toBeTruthy();
    });

    describe('on setted reload flag', () => {
      beforeEach(() => {
        service._setPostfachMailOnReload = jest.fn();
        service.shouldReload = true;
      });

      describe('and loaded vorgang resource', () => {
        it('should call set kommentar list reload', () => {
          service._handleVorgangChange(vorgangWithEingangStateResource);

          expect(service._setPostfachMailOnReload).toHaveBeenCalled();
        });

        it('and loaded vorgang resource should call set kommentar list reload', () => {
          service._handleVorgangChange(vorgangWithEingangStateResource);

          expect(service.shouldReload).toBeFalsy();
        });
      });
    });
  });
});
